@@ -1,8 +1,9 @@
 from dataclasses import dataclass
 from datetime import datetime
 from pathlib import Path
+from typing import List
 
-from subwinder._constants import TIME_FORMAT
+from subwinder._constants import REPO_URL, TIME_FORMAT
 from subwinder.exceptions import SubLibError
 from subwinder.lang import LangFormat, lang_3s
 
@@ -21,9 +22,15 @@ def build_media_info(data, dirname=None, filename=None):
 
     kind = data["MovieKind"]
     if kind in MEDIA_MAP:
-        return MEDIA_MAP[kind].from_data(data, dirname, filename)
+        media = MEDIA_MAP[kind].from_data(data)
+        media.set_dirname(dirname)
+        media.set_filename(filename)
+        return media
 
-    raise SubLibError(f"Undefined MovieKind: '{data['MovieKind']}'")
+    raise SubLibError(
+        f'The library encounterd an undefined MovieKind: "{kind}". You can raise an'
+        f" issue at the library repo to address this {REPO_URL}"
+    )
 
 
 @dataclass
@@ -67,8 +74,7 @@ class FullUserInfo(UserInfo):
         user_info = UserInfo.from_data(data)
 
         return cls(
-            id=user_info.id,
-            name=user_info.name,
+            **user_info.__dict__,
             rank=data["UserRank"],
             num_uploads=int(data["UploadCnt"]),
             num_downloads=int(data["DownloadCnt"]),
@@ -105,36 +111,55 @@ class MediaInfo:
     name: str
     year: int
     imdbid: str
-    dirname: Path
-    filename: Path
+    _dirname: Path
+    _filename: Path
+
+    def __init__(self, name, year, imdbid, dirname, filename):
+        self.name = name
+        self.year = year
+        self.imdbid = imdbid
+        self.set_dirname(dirname)
+        self.set_filename(filename)
 
     @classmethod
-    def from_data(cls, data, dirname=None, filename=None):
+    def from_data(cls, data):
         name = data["MovieName"]
+        # Sometimes this is returned as an `int`, sometimes it's a `str` ¯\_(ツ)_/¯
         year = int(data["MovieYear"])
         # For some reason opensubtitles sometimes returns this as an integer
         # Example: best guess when using `guess_media` for "the expanse" has
         #          `type(data["IDMovieIMDB"]) == int`
         imdbid = str(data.get("IDMovieImdb") or data["IDMovieIMDB"])
 
-        if dirname is not None:
-            dirname = Path(dirname)
-
-        if filename is not None:
-            filename = Path(filename)
-
-        return cls(name, year, imdbid, dirname, filename)
+        return cls(name, year, imdbid, dirname=None, filename=None)
 
     def set_filepath(self, filepath):
-        filepath = Path(filepath)
-        self.set_dirname(filepath.parent)
-        self.set_filename(filepath.name)
+        if filepath is None:
+            self.set_dirname(None)
+            self.set_filename(None)
+        else:
+            filepath = Path(filepath)
+
+            self.set_dirname(filepath.parent)
+            self.set_filename(filepath.name)
 
     def set_filename(self, filename):
-        self.filename = Path(filename)
+        self._filename = None if filename is None else Path(filename)
 
     def set_dirname(self, dirname):
-        self.dirname = Path(dirname)
+        self._dirname = None if dirname is None else Path(dirname)
+
+    def get_filepath(self):
+        if self.get_filename() is None or self.get_dirname() is None:
+            return None
+
+        return self.get_dirname() / self.get_filename()
+
+    def get_filename(self):
+        return self._filename
+
+    def get_dirname(self):
+        return self._dirname
 
 
 class MovieInfo(MediaInfo):
@@ -158,9 +183,14 @@ class EpisodeInfo(TvSeriesInfo):
     season: int
     episode: int
 
+    def __init__(self, name, year, imdbid, dirname, filename, season, episode):
+        super().__init__(name, year, imdbid, dirname, filename)
+        self.season = season
+        self.episode = episode
+
     @classmethod
-    def from_data(cls, data, dirname, filename):
-        tv_series = TvSeriesInfo.from_data(data, dirname, filename)
+    def from_data(cls, data):
+        tv_series = TvSeriesInfo.from_data(data)
         # Yay different keys for the same data!
         season = int(data.get("SeriesSeason") or data.get("Season"))
         episode = int(data.get("SeriesEpisode") or data.get("Episode"))
@@ -173,8 +203,8 @@ class EpisodeInfo(TvSeriesInfo):
             name=tv_series.name,
             year=tv_series.year,
             imdbid=tv_series.imdbid,
-            dirname=tv_series.dirname,
-            filename=tv_series.filename,
+            dirname=tv_series.get_dirname(),
+            filename=tv_series.get_filename(),
             season=season,
             episode=episode,
         )
@@ -245,9 +275,6 @@ class SubtitlesInfo:
     """
 
     size: int
-    num_downloads: int
-    num_comments: int
-    rating: float
     id: str
     file_id: str
     sub_to_movie_id: str
@@ -259,6 +286,8 @@ class SubtitlesInfo:
 
     @classmethod
     def from_data(cls, data):
+        # If the search was done with anything other than movie hash and size then
+        # there isn't a "IDSubMovieFile"
         if data["IDSubMovieFile"] == "0":
             sub_to_movie_id = None
         else:
@@ -266,15 +295,8 @@ class SubtitlesInfo:
 
         return cls(
             size=int(data["SubSize"]),
-            num_downloads=int(data["SubDownloadsCnt"]),
-            num_comments=int(data["SubComments"]),
-            # 0.0 is the listed rating if there are no ratings yet which seems deceptive
-            # at a glance
-            rating=None if data["SubRating"] == "0.0" else float(data["SubRating"]),
             id=data["IDSubtitle"],
             file_id=data["IDSubtitleFile"],
-            # If the search was done with anything other than movie hash and size then
-            # there isn't a "IDSubMovieFile"
             sub_to_movie_id=sub_to_movie_id,
             filename=Path(data["SubFileName"]),
             lang_2=data["ISO639"],
@@ -294,12 +316,73 @@ class SearchResult:
     media: MediaInfo
     subtitles: SubtitlesInfo
     upload_date: datetime
+    num_bad_reports: int
+    num_downloads: int
+    num_comments: int
+    rating: float
+    score: float
 
     @classmethod
-    def from_data(cls, data, dirname=None, filename=None):
-        author = None if data["UserID"] == "0" else UserInfo.from_data(data)
-        media = build_media_info(data, dirname, filename)
-        subtitles = SubtitlesInfo.from_data(data)
-        upload_date = datetime.strptime(data["SubAddDate"], TIME_FORMAT)
+    def from_data(cls, data):
+        return cls(
+            author=None if data["UserID"] == "0" else UserInfo.from_data(data),
+            media=build_media_info(data),
+            subtitles=SubtitlesInfo.from_data(data),
+            upload_date=datetime.strptime(data["SubAddDate"], TIME_FORMAT),
+            num_bad_reports=int(data["SubBad"]),
+            num_downloads=int(data["SubDownloadsCnt"]),
+            num_comments=int(data["SubComments"]),
+            # 0.0 is the listed rating if there are no ratings yet which seems deceptive
+            # at a glance
+            rating=None if data["SubRating"] == "0.0" else float(data["SubRating"]),
+            score=data["Score"],
+        )
 
-        return cls(author, media, subtitles, upload_date)
+
+@dataclass
+class GuessMediaResult:
+    """
+    Data container for a result from `AuthSubwinder`'s `guess_media` method
+    """
+
+    best_guess: MediaInfo
+    from_string: MediaInfo
+    from_imdb: List[MediaInfo]
+
+    @classmethod
+    def from_data(cls, data):
+        BEST_GUESS_KEY = "BestGuess"
+        FROM_STRING_KEY = "GuessMovieFromString"
+        IMDB_KEY = "GetIMDBSuggest"
+
+        # Deal with missing entries by filling with empty values
+        for key in [BEST_GUESS_KEY, FROM_STRING_KEY, IMDB_KEY]:
+            if key not in data:
+                data[key] = {}
+
+        # So just in case you're wondering why there's all these hoops to jump
+        # through. The API returns each value paired in a dict where the IMDB id is
+        # the key, but since we don't know the key we have to do some extra work to
+        # ignore it while still getting the value attached to it (Oh yeah, but
+        # "BestGuess" is **not** returned this way, yet all the others are)
+        best_guess = data[BEST_GUESS_KEY]
+
+        # So this one is a bit complicated, from what I've seen sometimes this is a
+        # `dict` where the key is the IMDB id, and sometimes its a `list` with length 1
+        from_string = data[FROM_STRING_KEY]
+        if type(from_string) == dict:
+            from_string = list(from_string.values())
+
+        if len(from_string) > 0:
+            from_string = from_string[0]
+
+        # When theres no results it's an empty `list`, when there are results it's a
+        # `dict` so need to force the potentially empty `list` to a `dict` first
+        from_imdb = list(dict(data[IMDB_KEY]).values())
+
+        return cls(
+            # Now that it's orgainzed build the appropriate `MediaInfo`
+            best_guess=build_media_info(best_guess) if best_guess else None,
+            from_string=build_media_info(from_string) if from_string else None,
+            from_imdb=[build_media_info(m) for m in from_imdb],
+        )
